package d1.core;

import java.io.Serializable;
import java.util.Date;
import java.util.UUID;

/**
 * An event in a calendar
 * This is a POJO that can be serialized and used in collections.
 * Only the id value is used in equality and hashtable, but date is used in sorting
 */
public class Event implements Serializable, Validatable, Comparable<Event> {


    private UUID id;
    private Date date;
    private String name;
    private String text;
    public static final String ERROR_NO_DATE = "No date: ";
    public static final String ERROR_NO_ID = "No ID: ";
    public static final String ERROR_NO_NAME = "No name: ";




    /**
     * empty constructor
     */
    public Event() {
    }

    /**
     * Full constructor.
     * Does not validate the values
     *
     * @param id
     * @param date
     * @param name
     * @param text
     */
    public Event(UUID id, Date date, String name, String text) {
        this.id = id;
        this.date = date;
        this.name = name;
        this.text = text;

    }

    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }

    public Date getDate() {
        return date;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    /**
     * Equality logic. Only the event ID is used for the logic;
     * any other attribute can change without affecting the test
     *
     * @param that other object
     * @return true iff this.id==that.id
     */
    public boolean equals(Object that) {
        if (this == that) {
            return true;
        }
        if (!(that instanceof Event)) {
            return false;
        }

        final Event event = (Event) that;

        if(id!=null) {
            return id.equals(event.id);
        } else {
            return false;
        }
    }

    /**
     * Hash code is derived from ID only.
     *
     * @return id-derived hash code
     */
    public int hashCode() {
        return id.hashCode();
    }

    /**
     * A valid event has an ID, timestamp and name. Text can be null
     *
     * @throws ValidationException if not valid
     */
    public boolean validate() throws ValidationException {
        if (id == null) {
            throw new ValidationException(ERROR_NO_ID + this);
        }
        if (name == null) {
            throw new ValidationException(ERROR_NO_NAME + this);
        }
        if (date == null) {
            throw new ValidationException(ERROR_NO_DATE + this);
        }
        return true;
    }

    /**
     * Convert to a string
     *
     * @return description of the event
     * MB: first example omits the line with the name in it
     */
    public String toString() {
        return "Event #" +
                id + " - "+
                (date != null ? date.toString() : "(no date)")
                +" - "
//comment the line out to simulate failures                
                + name + " - "
                + (text != null ? text : "");
    }

    /**
     * Sort order is defined by the sort order of dates.
     *
     * @param that
     * @return sort ordering of the items
     */
    public int compareTo(Event that) {
        return (date).compareTo(that.date);
    }

    /**
     * parse an iso date and convert to a datetime, then set the date
     * @param isodate
     * @throws IllegalArgumentException for a bad date
     */
    public void setDate(String isodate) {
        Date time = DateHelper.parseIsoDate(isodate);
        setDate(time);
    }

}
