package d1.core;

import java.io.Closeable;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.UUID;
import java.util.Date;

/**
 * A set of events.
 */
public class Events implements Validatable, Serializable,
        Iterable<Event> {


    public static String ERROR_NOT_CALENDAR_FILE = "File is not a calendar: ";

    /**
     * this is our store of events
     */
    private HashMap<UUID, Event> map = new HashMap<UUID, Event>();

    private String id = UUID.randomUUID().toString();;

    private String title="Calendar";

    private String author="antbook";

    public Events() {
    }

    /**
     * add an event
     *
     * @param event
     */
    public void add(Event event) {
        map.put(event.getId(), event);
        assert validate();
    }

    /**
     * delete an event
     *
     * @param event event to delete
     * @return true if the event was found
     */
    public boolean delete(Event event) {
        return delete(event.getId());
    }

    /**
     * delete an event
     *
     * @param eventID ID of event to delete
     * @return true if the event was found
     */
    public boolean delete(UUID eventID) {
        return map.remove(eventID) != null;
    }

    /**
     * Find an event with a given ID
     *
     * @param eventID
     * @return the event looked up, or null or none
     */
    public Event lookup(UUID eventID) {
        return map.get(eventID);
    }

    /**
     * an event set is valid if all events inside are valid
     *
     * @throws ValidationException if not valid
     */
    public boolean validate() throws ValidationException {
        for (Event event : this) {
            event.validate();
        }
        return true;
    }

    /**
     * create a human readable list of events
     *
     * @return a list of event instances in this class, one per line.
     */
    public String toString() {
        StringBuffer buffer = new StringBuffer(size() * 100);
        for (Event event : this) {
            buffer.append(event.toString());
            buffer.append('\n');
        }
        return buffer.toString();
    }

    /**
     * Save ourselves to a file
     *
     * @param filename
     * @throws IOException if a synchronous write failure occurred. Even if
     * we succeed, something may happen when the OS 
     * buffers are flushed.
     */
    public void save(String filename) throws IOException {
        ObjectOutputStream oos = null;
        FileOutputStream fos = null;
        try {
            fos = new FileOutputStream(filename);
            oos = new ObjectOutputStream(fos);
            oos.writeObject(this);
            oos.close();
        } finally {
            close(oos);
            close(fos);
        }
    }

    /**
     * size of the events
     *
     * @return #of event instances
     */
    public int size() {
        return map.size();
    }

    /**
     * Iterate over all events
     *
     * @return iterator of events
     */
    public Iterator<Event> iterator() {
        return map.values().iterator();
    }


    public String getId() {
        return id;
    }

    public void setId(String id) {
        this.id = id;
    }


    public String getTitle() {
        return title;
    }

    public void setTitle(String title) {
        this.title = title;
    }

    public String getAuthor() {
        return author;
    }

    public void setAuthor(String author) {
        this.author = author;
    }

    /**
     * @param filename file to load
     * @return the events loaded from file
     * @throws IOException
     */
    public static Events load(String filename) throws IOException {
        //no need to buffer the stream, as ObjectInputStream's helper classes
        //read in blocks
        ObjectInputStream ois = new ObjectInputStream(
                new FileInputStream(filename));
        try {
            //read in the object
            Events events = (Events) ois.readObject();
            //clean up
            ois.close();
            ois = null;
            //verify that the loaded types are valid
            events.validate();
            //return the new Events instance
            return events;
        } catch (ValidationException e) {
            IOException ioe = new IOException(
                    ERROR_NOT_CALENDAR_FILE + e.getMessage());
            ioe.initCause(e);
            throw ioe;
        } catch (ClassNotFoundException e) {
            //Very, very implausible: the data was read but the class absent
            IOException ioe = new IOException(ERROR_NOT_CALENDAR_FILE +
                    filename);
            ioe.initCause(e);
            throw ioe;
        } finally {
            //this cleanup only engages on premature exit, so
            //we always ignore exceptions raised. Something
            //is going wrong already.
            close(ois);
        }
    }

    /**
     * close a closeable, ignoring errors.
     *
     * @param toClose
     */
    private static void close(Closeable toClose) {
        if (toClose != null) {
            try {
                toClose.close();
            } catch (IOException ignored) {
                //ignore
            }
        }
    }

    /**
     * create an ID unique to this instance.
     * @return an event with new ID.
     */
    public synchronized Event createEvent() {
        Event event=new Event();
        event.setId(UUID.randomUUID());
        add(event);
        return event;
    }

}
