package d1.core.test;

import d1.core.Event;
import d1.core.Events;
import d1.core.Validatable;
import d1.core.ValidationException;
import junit.framework.TestCase;

import java.io.File;
import java.util.Date;
import java.util.UUID;

/**
 * Helper methods
 */
public abstract class CoreTestCase extends TestCase {

    protected CoreTestCase(String s) {
        super(s);
    }


    /**
     * Set the year to this year
     */
    public static final int YEAR = new Date().getYear();

    /**
     * assert that an object is valid
     *
     * @param target object to validate
     */
    public static void assertValid(Validatable target) {
        try {
            assertNotNull(target);
            target.validate();
        } catch (ValidationException e) {
            fail(e.getMessage());
        }
    }

    /**
     * assert that something is invalid
     *
     * @param target  item to test
     * @param message message that must be in the validation exception
     *                for success. This must not be null.
     */
    public static void assertInvalid(Validatable target, String message) {
        try {
            assertNotNull(target);
            target.validate();
            fail("Expected validation failure with " + message);
        } catch (ValidationException e) {
            assertTrue("Expected exception containing " +
                    message
                    + " but got " + e.getMessage(),
                    e.getMessage().indexOf(message) >= 0);
        }
    }

    /**
     * "Spring" equinox, first one of the year.
     *
     * @param id id to give the event
     * @return a configured event
     */
    public Event createSpringEquinox(UUID id) {
        return new Event(id,
                new Date(YEAR, 3, 21),
                "Spring Equinox",
                "Day and Night of equal duration");
    }


    /**
     * Autumn Equinox
     *
     * @param id id to give the event
     * @return a configured event
     */
    public Event createAutumnEquinox(UUID id) {
        return new Event(id,
                new Date(YEAR, 9, 21),
                "Autumn Equinox",
                "Day and Night of equal duration");
    }

    /**
     * Northern Midsummer
     *
     * @param id
     * @return new event
     * @todo check
     */
    public Event createMidsummer(UUID id) {
        return new Event(id,
                new Date(YEAR, 6, 21),
                "Midsummer, june 21/22",
                "Longest day of the year for the northern hemisphere" +
                "longest night for the southern half of the world");
    }

    /**
     * Hogmanay, scottish for New Year's eve 31-dec-????
     *
     * @param id
     * @return new event
     */
    public Event createHogmanay(UUID id) {
        return new Event(id,
                new Date(YEAR, 12, 31),
                "Hogmanay",
                "New Year's Eve");
    }

    /**
     * create a calendar of all valid events
     *
     * @return a calendar with events in it
     */
    public Events createCalendar() {
        int id = 1;
        Events calendar = new Events();
        calendar.add(createMidsummer(newID()));
        calendar.add(createSpringEquinox(newID()));
        calendar.add(createAutumnEquinox(newID()));
        calendar.add(createHogmanay(newID()));
        return calendar;
    }

    public UUID newID() {
        return UUID.randomUUID();
    }

    /**
     * get a value passed into the JVM
     *
     * @param property property to look for
     * @return the value of the system property
     * @throws AssertionError if the property is undefined
     */
    public static String getProperty(String property) {
        String value = System.getProperty(property);
        assertNotNull("Property " + property + " is undefined", value);
        return value;
    }

    /**
     * get the name of a file from a property
     *
     * @param property to look up
     * @return the file
     * @throws AssertionError if the property is undefined, or the file does not
     *                        exist
     */
    public static File getFileProperty(String property) {
        String value = getProperty(property);
        File file = new File(value);
        assertTrue("File " + file + " does not exist", file.exists());
        return file;
    }

    /**
     * Get a boolean, bail out if it is missing
     *
     * @param property
     * @return
     */
    public static boolean getBooleanProperty(String property) {
        String value = getProperty(property);
        return Boolean.valueOf(value).booleanValue();
    }

    /**
     * @param property
     * @return the integer value of a property
     * @throws NumberFormatException if the property wont parse
     */
    public static int getIntProperty(String property) {
        String value = getProperty(property);
        return Integer.valueOf(value).intValue();
    }

    /**
     * assert that two objects are equal in equals(),
     * hashode() and compareTo() operations. This
     *
     * @param first  first comparable to test
     * @param second second comparable to test
     */
    public static void assertComparableObjectsEqual(Comparable first,
                                                Comparable second) {
        assertVeryEqual(first, second);
        assertEquals("first.compareTo(second)!=0", 0, first.compareTo(second));
        assertEquals("second.compareTo(first)!=0", 0, second.compareTo(first));
    }

    /**
     * assert that hashCode() and equals() ; are consistent
     *
     * @param first  first object
     * @param second second object that must equal first
     */
    public static void assertVeryEqual(Object first, Object second) {
        assertEquals(first, second);
        //reflexive rule
        assertEquals("reflexive equality failed", second, first);
        assertEquals("Hash codes must match",
                first.hashCode(), second.hashCode());
    }

    /**
     * assert that two objects are unequal in equals(),
     * hashode() and compareTo() operations.
     *
     * @param first
     * @param second
     */
    public static void assertComparableObjectsNotEqual(Comparable first,
                                                   Comparable second) {
        assertNotSame(first, second);
        assertFalse("Objects appear equal",
                first.equals(second));
        assertFalse("Different objects must not be equal",
                second.equals(first));
        assertTrue("Hash codes must not match",
                first.hashCode() != second.hashCode());
        assertTrue("compareTo must be different",
                first.compareTo(second) != 0);
    }
}
