package d1.persist.hibernate;

import d1.persist.Event;
import d1.persist.session.AbstractCalendarSession;
import d1.persist.session.CalendarSession;
import d1.persist.session.CalendarSessionFactory;
import org.hibernate.HibernateException;
import org.hibernate.Query;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;

import java.util.List;

/**
 * representation of a set of events.
 * This one uses the hibernate API, not the EJB3.0 ones. It is the easiest
 * one to get working, and the only one that works outside an application server.
 *
 */
public class HibernateSession extends AbstractCalendarSession
        implements CalendarSession, CalendarSessionFactory {


    private SessionFactory factory;

    public HibernateSession(SessionFactory factory) {
        this.factory = factory;
    }

    public SessionFactory getFactory() {
        return factory;
    }

    /**
     * Return ourselves as a calendar session
     *
     * @return this instance.
     */
    public CalendarSession getSession() {
        return this;
    }

    /**
     * Get the events that match a criteria.
     *
     * @param where search string or null for all events
     *
     * @return a list of all events matching the pattern
     */
    public List<Event> getEvents(String where) {
        String queryString = makeSelection(where);
        Session session = beginSession();
        try {
            Query query = session.createQuery(
                    queryString);
            return (List<Event>) query.list();
        } finally {
            session.close();
        }
    }



    /**
     * Get the current set of events in the database
     * @param where
     * @return the event count
     */
    public int getEventCount(String where) {
        String filter = makeSelection(where);
        return ((Integer) executeUniqueQuery("select count(event) " + filter));
    }

    private Object executeUniqueQuery(String queryString) {
        Session session = beginSession();
        Object result=null;
        try {
            Query query = session.createQuery(queryString);
            result = query.uniqueResult();
        } finally {
            session.close();
        }
        return result;
    }


    /**
     * Get the number of events in the database
     *
     * @return the current event count
     */
    public int getEventCount() {
        return ((Integer) executeUniqueQuery("select count(event)"));
    }


    /**
     * add a new event or update an existing one
     *
     * @param event
     */
    public void addOrUpdate(Event event) {
        Session session = beginSession();
        Transaction transaction = session.beginTransaction();
        try {
            session.saveOrUpdate(event);
            transaction.commit();
        } catch(HibernateException e) {
            transaction.rollback();
            throw e;
        } finally {
            session.close();
        }
    }


    /**
     * Look up an event by key.
     *
     * @param key ID of event
     *
     * @return event or null for no match
     */
    public Event findEvent(String key) {
        Session session = beginSession();
        try {
            Query query = session.createQuery(
                    "from Event e where e.key = :key");
            query.setString("key", key);
            List result = query.list();
            if (result.size() < 1) {
                return null;
            }
            return (Event) result.get(0);
        } finally {
            session.close();
        }
    }

    /**
     * Remove an event given by a key Do nothing if it is not present.
     *
     * @param key
     * @return true if it was found and removed
     */
    public boolean removeEvent(String key) {
        Session session = beginSession();
        Transaction transaction = session.beginTransaction();
        try {
            Event loaded = (Event) session.load(Event.class, key);
            if (loaded != null) {
                session.delete(loaded);
            }
            transaction.commit();
            return loaded!=null;
        } finally {
            session.close();
        }
    }

    /**
     * begin a new session. Remember to close it afterwards
     *
     * @return a new session
     */
    private Session beginSession() {
        return factory.openSession();
    }

    /**
     * at terminate time the session factory is closed
     */
    public synchronized void closeFactory() {
        getFactory().close();
    }

/*
    public static void bindToPropertyConfiguration() {
        SessionFactory sessionFactory;
        String resource;

        //get the JVM properties and act off them, if set
        Properties sysProps = System.getProperties();
        Properties actualProps =
            PersistTools.extractOptionsWithPrefix(sysProps, "db.");
        if (sysProps.getProperty(LifecycleListener.PROP_DYNAMIC) != null) {
            //dynamic resource takes priority over everything
            resource = PersistTools.HIBERNATE_DYNAMIC_RESOURCE;
        } else if (LifecycleListener.USE_MYSQL) {
            //mysql
            resource = PersistTools.HIBERNATE_MYSQL_RESOURCE;
        } else {
            //hibernate
            resource = PersistTools.HIBERNATE_HSQLDB_RESOURCE;
        }
        sessionFactory = PersistTools.createSessionFactory(
            resource,
            actualProps);
        bindToDatabase(sessionFactory);
    }*/
}
