package d1.persist.hibernate;

import org.hibernate.SessionFactory;
import org.hibernate.cfg.Environment;
import org.hibernate.cfg.AnnotationConfiguration;
import org.hibernate.cfg.Configuration;

import java.util.Properties;

import d1.persist.Event;

/**

 */
public final class HibernateTools {
    /**
     * The static session factory is demand-initialised
     */

    private static SessionFactory sessionFactory;
    /**
     * The name of an entry in persistence.xml.
     * {@value}
     */
    public static final String HIBERNATE_MANAGER = "hibernate";
    /**
     * Name of the resource with the HSQL config
     * {@value}
     */
    public static final String HIBERNATE_HSQLDB_RESOURCE =
            "d1/persist/config/hibernate-hsql.cfg.xml";
    /**
     * Name of the resource with the MYSQL config {@value}
     */
    public static final String HIBERNATE_MYSQL_RESOURCE =
            "d1/persist/config/hibernate-mysql.cfg.xml";
    /** Name of the resource with the MYSQL config {@value} */
    public static final String HIBERNATE_DYNAMIC_RESOURCE =
        "d1/persist/config/hibernate-dynamic.cfg.xml";

    private HibernateTools() {
    }

    /**
     * Get a session factory configured with all the classes that
     * this project can persist.
     * @return the existing or new session factory
     * @throws RuntimeException if there is something wrong with the local configuration,
     * and we cannot create a session factory.
     */
    public static synchronized SessionFactory getSessionFactory() {
        if(sessionFactory==null) {
            //demand create the session
            buildSessionFactory();
        }
        return sessionFactory;
    }

    /**
     * Internal method to actually build the session factory.
     */
    private static synchronized void buildSessionFactory() {
        Properties props=new Properties();
        props.put(Environment.DIALECT, HibernateConstants.DIALECT_HSQLDB);
        sessionFactory = createSessionFactory(props);
    }

    /**
     * Create a session factory with the given set of options.
     * @param options any extra options, can be null
     * @return a fully configured session factory
     */
    public static SessionFactory createSessionFactory(Properties options) {
        AnnotationConfiguration config = new AnnotationConfiguration();
        config.addAnnotatedClass(Event.class);
        return finishConfigurationAndCreateFactory(config, options);
    }

    /**
     * Create a session factory from a resource file
     * @param resource resource to load
     * @param options any extra options, can be null
     * @return a fully configured session factory
     */
    public static SessionFactory createSessionFactory(String resource,
                                                      Properties options) {
        Configuration config = new AnnotationConfiguration();
        config.configure(resource);
        if (options != null) {
            config.addProperties(options);
        }
        return config.buildSessionFactory();
    }

    /**
     * finish off all preparation work and create the factory
     * @param config
     * @param options
     * @return
     */
    private static SessionFactory finishConfigurationAndCreateFactory(
            AnnotationConfiguration config,
            Properties options)
    {
        if (options != null) {
            config.addProperties(options);
        }
        return config.buildSessionFactory();
    }
}
