package d1.persist.test.hibernate;

import d1.persist.Event;
import d1.persist.hibernate.HibernateTools;
import org.hibernate.ObjectDeletedException;
import org.hibernate.Query;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.Session;

import java.util.List;

import junit.framework.Assert;

/** Test that in a session we can do things */
public class HibernateSessionTest extends HibernateTestBase {

    public HibernateSessionTest(String s) {
        super(s);
    }

    /**
     * created on demand in the setup helper. Unless of course it wont create,
     * in which case every setup cal
     */
    private static SessionFactory sessionFactory;
    private Session session;

    /** create the session factory */
    protected void setUp() throws Exception {
        super.setUp();
        if (sessionFactory == null) {
            sessionFactory = HibernateTools.createSessionFactory(getSettings());
        }
        openSession();
    }

    /**
     * Tears down the fixture, for example, close a network connection. This
     * method is called after a test is executed.
     */
    protected void tearDown() throws Exception {
        super.tearDown();
        closeSession();
    }

    private synchronized void closeSession() {
        closeSession(session);
        session = null;
    }


    private synchronized void openSession() {
        SessionFactory factory = getSessionFactory();
        session = factory.openSession();
    }

    /**
     * Close a session if it is null
     *
     * @param toclose
     */
    protected void closeSession(Session toclose) {
        if (toclose != null) {
            toclose.close();
        }
    }

    /**
     * Get the current session factory
     *
     * @return
     */
    public static SessionFactory getSessionFactory() {
        return sessionFactory;
    }


    public void testNoTransactionNeeded() throws Exception {

        Event event = createTestEvent();
        session.persist(event);
    }

    /**
     * Tests that updating our event (and flushing it) increases the version
     *
     * @throws Exception
     */
    public void testVersionIncrements() throws Exception {
        Event event = createTestEvent();
        session.persist(event);
        session.flush();
        int version = event.getVersion();
        event.setText("updated");
        session.flush();
        session.refresh(event);
        Assert.assertFalse(version == event.getVersion());
    }


    /**
     * A non-transacted save/restore
     *
     * @throws Exception
     */
    public void testPersistAndRetrieve() throws Exception {
        Event event = createTestEvent();
        session.persist(event);
        closeSession();
        assertEventIsInDB(event);
    }

    /**
     * Save an event in a transaction
     *
     * @throws Exception
     */
    public void testTransactedPersistAndRetrieve() throws Exception {
        Event event = createTestEvent();
        Transaction transaction = session.beginTransaction();
        try {
            session.persist(event);
            transaction.commit();
        } catch (Exception e) {
            transaction.rollback();
            throw e;
        }
        closeSession();
        assertEventIsInDB(event);
    }

    public void testDetachedObject() throws Exception {
        Event event = createTestEvent();
        session.save(event);
        session.flush();
        closeSession();

        //here the event is detached.
        //we change it outside an operation.
        String text = "modified";
        event.setText(text);

        session = getSessionFactory().openSession();
        Event merged = (Event) session.merge(event);
        Assert.assertEquals(event, merged);
        session.flush();
        closeSession();
        session = getSessionFactory().openSession();
        Event loaded = (Event) session.load(Event.class, event.getKey());
        Assert.assertEquals(text, loaded.getText());
        Assert.assertEquals(event, loaded);
    }

    public void testQuery() throws Exception {
        Event event = createTestEvent();
        session.persist(event);
        Query query = session.createQuery(
                "from Event e where e.key = :key");
        query.setString("key", event.getKey());
        List result = query.list();
        Assert.assertNotNull(result);
        Assert.assertEquals("expected only 1 item in query result",
                1, result.size());
        Event e2 = (Event) result.get(0);
        Assert.assertEquals(event, e2);
    }

    public void testDeleteAndFind() throws Exception {

        Event event = createTestEvent();
        String key = event.getKey();
        Transaction transaction = session.beginTransaction();
        try {
            session.persist(event);
            session.delete(event);
            Event loaded = null;
            try {
                loaded = (Event) session.load(Event.class, key);
                Assert.fail("should have got an error here");
            } catch (ObjectDeletedException e) {
                //expected
            }
        } finally {
            transaction.rollback();
        }
    }

    /**
     * assert that an event is in the database
     *
     * @param event
     */
    protected void assertEventIsInDB(Event event) {
        Session newSession;
        newSession = sessionFactory.openSession();
        try {
            Event loaded = (Event) newSession.load(Event.class,
                    event.getKey());
            Assert.assertEquals(event, loaded);
        } finally {
            newSession.close();
        }
    }
}
