package d1.persist;

import d1.core.DateHelper;
import d1.core.Validatable;
import d1.core.ValidationException;

import java.io.Serializable;
import java.util.Date;
import java.util.UUID;
import javax.persistence.*;

/** This is a persistent event. */
@Entity
@Table(name = "EVENTS")
public class Event implements Serializable,
    Validatable, Comparable<Event> {


    /** unique ID */

    private UUID id;

    /** event created */
    private Date date;
    /** event name length="255" */
    private String name;
    /** text contents of an event */
    private String text;
    /** version is used for hibernate optimistic locks */

    private int version = -1;
    /** last modified flag. Can be used in the RSS feed. */
    private Date lastModified;
    public static final String ERROR_NO_DATE = "No date: ";
    public static final String ERROR_NO_ID = "No ID: ";
    public static final String ERROR_NO_NAME = "No name: ";

    /** empty constructor creates an event with a new ID */
    public Event() {

    }

    /**
     * More detailed constructor. Does not validate the values
     *
     * @param id
     * @param date
     * @param name
     * @param text
     */
    public Event(UUID id, Date date, String name, String text) {
        this.id = id;
        this.date = date;
        this.name = name;
        this.text = text;
    }

    public Event(Event that) {
        id = that.id;
        date = that.date;
        name = that.name;
        text = that.text;
        version = that.version;
        lastModified = that.lastModified;
    }

    /** attach a new ID/key to the item. */
    public void newId() {
        id = UUID.randomUUID();
    }

    /**
     * To make sure that UUIDs persist as strings, there are special
     * getters/setters that access the field as a string, hence creating a
     * property apparently of string type. However, secretly, they are turned
     * into UUID instances
     *
     * @return the guid
     */
    @Id
    @Column(name = "EVENT_ID", length = 48, unique = true)
    public String getKey() {
        return id.toString();
    }

    /**
     * Convert the ID to a string and process
     *
     * @param id
     */
    public void setKey(String id) {
        this.id = UUID.fromString(id);
    }

    /**
     * The underlying Id property is marked as transient
     *
     * @return
     */
    @Transient
    public UUID getId() {
        return id;
    }

    public void setId(UUID id) {
        this.id = id;
    }


    /**
     * @see <a href="http://www.hibernate.org/hib_docs/ejb3-api/javax/persistence/Temporal.html">EJB3</a>
     * @return
     */
    @Column(name = "EVENT_DATE")
    @Temporal(value = TemporalType.TIMESTAMP)
    public Date getDate() {
        return date;
    }

    public void setDate(Date date) {
        this.date = date;
    }

    @Column(name = "EVENT_NAME")
    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    @Column(name = "EVENT_TEXT",
            length = 8192)
    public String getText() {
        return text;
    }

    public void setText(String text) {
        this.text = text;
    }

    @Version
    @Column(name = "EVENT_VERSION")
    public int getVersion() {
        return version;
    }

    public void setVersion(int version) {
        this.version = version;
    }

    @Temporal(value = TemporalType.TIMESTAMP)
    @Column(name = "EVENT_LASTMODIFIED")
    public Date getLastModified() {
        return lastModified;
    }

    public void setLastModified(Date lastModified) {
        this.lastModified = lastModified;
    }

    /**
     * Equality logic. Only the event ID is used for the logic; any other
     * attribute can change without affecting the test
     *
     * @param that other object
     * @return true iff this.id==that.id
     */
    public boolean equals(Object that) {
        if (this == that) {
            return true;
        }
        if (!(that instanceof Event)) {
            return false;
        }

        final Event event = (Event) that;
        if (id == null) {
            return event.getId() == null;
        }
        return id.equals(event.getId());
    }

    /**
     * Hash code is derived from ID only.
     *
     * @return id-derived hash code
     */
    public int hashCode() {
        return id.hashCode();
    }

    /**
     * A valid event has an ID, timestamp and name. Text can be null
     *
     * @throws ValidationException if not valid
     */
    public boolean validate() throws ValidationException {
        if (id == null) {
            throw new ValidationException(ERROR_NO_ID + this);
        }
        if (name == null) {
            throw new ValidationException(ERROR_NO_NAME + this);
        }
        if (date == null) {
            throw new ValidationException(ERROR_NO_DATE + this);
        }
        return true;
    }

    /**
     * Convert to a string
     *
     * @return description of the event MB: first example omits the line with
     *         the name in it
     */
    public String toString() {
        return "Event #" +
                id + " - " +
                (date != null ? date.toString() : "(no date)")
                + " - "
                + name + " - "
                + (text != null ? text : "");
    }

    /**
     * Sort order is defined by the sort order of dates.
     *
     * @param that
     * @return sort ordering of the items
     */
    public int compareTo(Event that) {
        return (id).compareTo(that.id);
    }

    /**
     * parse an iso date and convert to a datetime, then set the date
     *
     * @param isodate
     * @throws IllegalArgumentException for a bad date
     */
    public void setDate(String isodate) {
        Date time = DateHelper.parseIsoDate(isodate);
        setDate(time);
    }


}

