package d1.persist.session;

import d1.persist.Event;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.EntityTransaction;
import javax.persistence.Query;
import java.util.List;

/**
 * representation of a set of events
 *
 * This is the EJB3.0 implementation that uses app managed persistences
 */
public class StandaloneCalendar
        extends AbstractCalendar implements CalendarSession, CalendarFactory {


    private EntityManagerFactory factory;

    public StandaloneCalendar(EntityManagerFactory factory) {
        this.factory = factory;
    }

    public synchronized EntityManagerFactory getFactory() {
        return factory;
    }

    /**
     * Return ourselves as a calendar session
     *
     * @return this instance.
     */
    public CalendarSession getSession() {
        return this;
    }


    /**
     * Get the likely cause of failure.
     *
     * @return details to include in error messages.
     */
    public String getLikelyCauseOfFailure() {
        return "EJB entity manager factory playing up";
    }

    /**
     * Close the factory. Called at the end of life of the system, though this
     * cannot be guaranteed.
     */
    public synchronized void closeFactory() {
        if (factory != null) {
            try {
                factory.close();
            } finally {
                factory = null;
            }
        }
    }

    /**
     * Get the events that match a criteria.
     *
     * @param where search string or null for all events
     *
     * @return a list of all events matching the pattern
     */
    @SuppressWarnings({"unchecked"})
    public List<Event> getEvents(String where) {
        String queryString = makeSelection(where);
        EntityManager manager = createEntityManager();
        try {
            Query query = manager.createQuery(
                    queryString);
            return (List<Event>) query.getResultList();
        } finally {
            manager.close();
        }
    }



    /**
     * Get the number of events in the database
     * @param where criteria
     * @return the current event count
     */
    public int getEventCount(String where) {
        EntityManager manager = createEntityManager();
        try {
            String queryString = makeSelection(where);
            Query query = manager.createQuery("SELECT COUNT(event) " + queryString);
            Object result = query.getSingleResult();
            return ((Integer) result);
        } finally {
            manager.close();
        }
    }

    /**
     * add a new event or update an existing one
     *
     * @param event event to update
     */
    public void addOrUpdate(Event event) {
        EntityManager manager = createEntityManager();
        try {
            EntityTransaction transaction = manager.getTransaction();
            transaction.begin();
            manager.persist(event);
            transaction.commit();
        } finally {
            manager.close();
        }
    }


    /**
     * Look up an event by key.
     *
     * @param key ID of event
     *
     * @return event or null for no match
     */
    public Event findEvent(String key) {
        EntityManager manager = createEntityManager();
        try {
            Query query = manager.createQuery(
                    "SELECT e from Event e where e.key = :key");
            query.setParameter("key", key);
            List result = query.getResultList();
            if (result.size() < 1) {
                return null;
            }
            return (Event) result.get(0);
        } finally {
            manager.close();
        }
    }

    /**
     * Remove an event given by a key Do nothing if it is not present.
     *
     * @param key event key
     *
     * @return true if it was found and removed
     */
    public boolean removeEvent(String key) {
        EntityManager manager = createEntityManager();
        try {
            EntityTransaction transaction = manager.getTransaction();
            transaction.begin();
            Event loaded = manager.find(Event.class, key);
            if (loaded != null) {
                manager.remove(loaded);
            }
            transaction.commit();
            return loaded!=null;
        } finally {
            manager.close();
        }
    }

    /**
     * begin a new session. Remember to close it afterwards
     *
     * @return a new session
     */
    private synchronized EntityManager createEntityManager() {
        return factory.createEntityManager();
    }


}
