package d1.persist.test;

import d1.core.Validatable;
import d1.core.ValidationException;
import d1.persist.Event;
import junit.framework.TestCase;

import java.io.File;
import java.util.Date;
import java.util.Properties;
import java.util.UUID;


/**
 * Base class for testing persistence stuff.
 * This is derived from d1.core.test.CoreTestCase; the reason for copying it
 * was simply to avoid having the code here depend upon the test code from
 * the other project.
 */
public abstract class PersistenceTestBase extends TestCase {
    private static int counter = 0;
    protected Properties settings;

    protected PersistenceTestBase(String s) {
        super(s);
    }


    /**
     * Set the year to this year
     */
    public static final int YEAR = new Date().getYear();

    /**
     * assert that an object is valid
     *
     * @param target object to validate
     */
    public static void assertValid(Validatable target) {
        try {
            assertNotNull(target);
            target.validate();
        } catch (ValidationException e) {
            fail(e.getMessage());
        }
    }

    /**
     * assert that something is invalid
     *
     * @param target  item to test
     * @param message message that must be in the validation exception
     *                for success. This must not be null.
     */
    public static void assertInvalid(Validatable target, String message) {
        try {
            assertNotNull(target);
            target.validate();
            fail("Expected validation failure with " + message);
        } catch (ValidationException e) {
            assertTrue("Expected exception containing " +
                    message
                    + " but got " + e.getMessage(),
                    e.getMessage().indexOf(message) >= 0);
        }
    }

    public UUID newID() {
        return UUID.randomUUID();
    }


    /**
     * get a value passed into the JVM
     *
     * @param property property to look for
     * @return the value of the system property
     * @throws AssertionError if the property is undefined
     */
    public static String getProperty(String property) {
        String value = System.getProperty(property);
        assertNotNull("Property " + property + " is undefined", value);
        return value;
    }

    /**
     * get the name of a file from a property
     *
     * @param property to look up
     * @return the file
     * @throws AssertionError if the property is undefined, or the file does not
     *                        exist
     */
    public static File getFileProperty(String property) {
        String value = getProperty(property);
        File file = new File(value);
        assertTrue("File " + file + " does not exist", file.exists());
        return file;
    }

    /**
     * Get a boolean, bail out if it is missing
     *
     * @param property
     * @return
     */
    public static boolean getBooleanProperty(String property) {
        String value = getProperty(property);
        return Boolean.valueOf(value).booleanValue();
    }

    /**
     * @param property
     * @return the integer value of a property
     * @throws NumberFormatException if the property wont parse
     */
    public static int getIntProperty(String property) {
        String value = getProperty(property);
        return Integer.valueOf(value).intValue();
    }

    /**
     * assert that two objects are equal in equals(),
     * hashode() and compareTo() operations. This
     *
     * @param first  first comparable to test
     * @param second second comparable to test
     */
    public static void assertComparableObjectsEqual(Comparable first,
                                                    Comparable second) {
        assertVeryEqual(first, second);
        assertEquals("first.compareTo(second)!=0", 0, first.compareTo(second));
        assertEquals("second.compareTo(first)!=0", 0, second.compareTo(first));
    }

    /**
     * assert that hashCode() and equals() ; are consistent
     *
     * @param first  first object
     * @param second second object that must equal first
     */
    public static void assertVeryEqual(Object first, Object second) {
        assertEquals(first, second);
        //reflexive rule
        assertEquals("reflexive equality failed", second, first);
        assertEquals("Hash codes must match",
                first.hashCode(), second.hashCode());
    }

    /**
     * assert that two objects are unequal in equals(),
     * hashode() and compareTo() operations.
     *
     * @param first
     * @param second
     */
    public static void assertComparableObjectsNotEqual(Comparable first,
                                                       Comparable second) {
        assertNotSame(first, second);
        assertFalse("Objects appear equal",
                first.equals(second));
        assertFalse("Different objects must not be equal",
                second.equals(first));
        assertTrue("Hash codes must not match",
                first.hashCode() != second.hashCode());
        assertTrue("compareTo must be different",
                first.compareTo(second) != 0);
    }

    /**
     * "Spring" equinox, first one of the year.
     *
     * @param id id to give the event
     * @return a configured event
     */
    public Event createSpringEquinox(UUID id) {
        return new Event(id,
                new Date(YEAR, 3, 21),
                "Spring Equinox",
                "Day and Night of equal duration");
    }

    /**
     * Autumn Equinox
     *
     * @param id id to give the event
     * @return a configured event
     */
    public Event createAutumnEquinox(UUID id) {
        return new Event(id,
                new Date(YEAR, 9, 21),
                "Autumn Equinox",
                "Day and Night of equal duration");
    }

    /**
     * Create a test event for our entertainment
     * @return a new event. Every one has different id, name and text. Date will change,
     * but only with the granularity of the clock.
     */
    protected Event createTestEvent() {
        counter++;
        Event event = new Event();
        event.newId();
        Date date = new Date();
        event.setDate(date);
        event.setLastModified(date);
        event.setName("Event #" + counter);
        event.setText("test event #" + counter + " created at " + date);
        return event;
    }

    /**
     * Sets up the fixture, for example, open a network connection.
     * This method is called before a test is executed.
     */
    protected void setUp() throws Exception {
        super.setUp();
        settings = createSettings();

    }

    /**
     * Create a settings map
     * @return a map filled in with whatever we want
     */
    public static Properties createSettings() {
        Properties properties = new Properties();
        return properties;
    }

    public Properties getSettings() {
        return settings;
    }
}
